/*
  Copyright (c) 2010 Werner Riese 

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#include "FlyMode.h"
 
 
FlyMode::FlyMode(QObject *parent, const QVariantList &args)
        : Plasma::Wallpaper(parent, args) {

    QDesktopWidget* desktop = QApplication::desktop();
    
    _screenWidth  = desktop->width();
    _screenHeight = desktop->height();

    _timer  = new QTimer(this);
    _pixmap = new QPixmap();   

    _directionVertical = true;     
    _pixelPerSecond    = SPEED_DEFAULT;
    _pictureOffset     = 0;   
    _wallpaper         = "/usr/share/wallpapers/Air/contents/images/1920x1200.jpg";
          
    _stopPaint    = false;
    _pixmapLoaded = false;

    connect(_timer, SIGNAL(timeout()), this, SLOT(updateWallpaper()));
    connect(this, SIGNAL(urlDropped(KUrl)), this, SLOT(setWallpaper(KUrl)));   

    _timer->start((int)(1000 / FRAMES_PER_SECOND));
}
 
 
FlyMode::~FlyMode() {
}  


QWidget* FlyMode::createConfigurationInterface(QWidget* parent) {
 
    // the returned configWidget of this function is deleted by the caller
    QWidget* _configWidget = new QWidget(parent);    
    
    _label1       = new QLabel(i18n("Seamless Picture or Pattern :"));
    _kurl         = new KUrlRequester();
    _comboBox     = new QComboBox();
    _label2       = new QLabel(i18n("speed : "));
    _slider       = new QSlider(Qt::Horizontal);
    
    _kurl->setText(_wallpaper);
    
    _comboBox->addItem(QString(i18n("move vertically")));
    _comboBox->addItem(QString(i18n("move horizontally")));
    _comboBox->setCurrentIndex( (_directionVertical) ? 0 : 1 );
    
    _slider->setMinimum(1);
    _slider->setMaximum(SPEED_STEP_COUNT);
    _slider->setSingleStep(1);
    _slider->setPageStep(1);
    _slider->setTickInterval(1);
    _slider->setTickPosition(QSlider::TicksBelow);
    _slider->setValue(_pixelPerSecond / SPEED_STEP);
   
    QVBoxLayout* _layoutMain = new QVBoxLayout();
    QHBoxLayout* _layout1    = new QHBoxLayout();
    QHBoxLayout* _layout2    = new QHBoxLayout();

    _layout1->addWidget(_comboBox);
    _layout1->addStretch();
    
    _layout2->addWidget(_label2);
    _layout2->addWidget(_slider);
    
    _layoutMain->addWidget(_label1);
    _layoutMain->addWidget(_kurl);
    _layoutMain->addLayout(_layout1);
    _layoutMain->addLayout(_layout2);
    _layoutMain->addStretch();
    
    _configWidget->setLayout(_layoutMain);

    connect(_kurl, SIGNAL(textChanged(QString)), this, SLOT(setWallpaper(QString)));
    connect(_kurl, SIGNAL(returnPressed(QString)), this, SLOT(setWallpaper(QString)));
    connect(_kurl, SIGNAL(urlSelected(KUrl)), this, SLOT(setWallpaper(KUrl)));

    connect(_comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setDirection(int)));
    connect(_comboBox, SIGNAL(activated(int)), this, SLOT(setDirection(int)));
    connect(_slider, SIGNAL(valueChanged(int)), this, SLOT(setSpeed(int)));
    connect(_slider, SIGNAL(sliderMoved(int)), this, SLOT(setSpeed(int)));
    
    connect(this, SIGNAL(settingsChanged(bool)), parent, SLOT(settingsChanged(bool)));

    return _configWidget;
}
    

void FlyMode::save(KConfigGroup &config) {
    
    config.writeEntry("wallpaper", _wallpaper);
    config.writeEntry("speed",     QVariant(_pixelPerSecond));                     
    config.writeEntry("direction", QVariant(_directionVertical));    
}


void FlyMode::init(const KConfigGroup &config) {

    _timer->stop();
    
    _wallpaper         = config.readEntry("wallpaper", "/usr/share/wallpapers/Air/contents/images/1920x1200.jpg");
    _directionVertical = config.readEntry("direction", QVariant(true)).toBool();    
    _pixelPerSecond    = config.readEntry("speed", QVariant(SPEED_DEFAULT)).toInt();                     
    _pixelPerSecond    = (_pixelPerSecond < SPEED_MIN || _pixelPerSecond > SPEED_MAX) ? SPEED_DEFAULT : _pixelPerSecond;
   
    _pixmapLoaded = false;
    
    setWallpaper(_wallpaper);

    _timer->start();
}


void FlyMode::updateWallpaper() {
    
    // shift seamless picture back to start
    _pictureOffset += (float)_pixelPerSecond / FRAMES_PER_SECOND;

    if( _pictureOffset >= 0 ) 
        _pictureOffset -= (_directionVertical) ? _pixmap->height() : _pixmap->width();   
    
    emit update(boundingRect());  
}


void FlyMode::setWallpaper(const QString& url) {
    
    if( !QFile::exists(url) ) 
        return;  
    
    if( (_wallpaper == url) && _pixmapLoaded )
        return;

    _stopPaint = true;
    
    int prevSize = (_directionVertical) ? -_pixmap->height() : -_pixmap->width();   

    _wallpaper = url;

    _pixmap->load(_wallpaper);     
    _pixmapLoaded = true;
 
    int newSize = (_directionVertical) ? -_pixmap->height() : -_pixmap->width();   

    // avoid image jump by reseting the offset
    if( newSize != prevSize )
        _pictureOffset = newSize;
    
    _stopPaint = false;
    
    emit configNeedsSaving();
    emit settingsChanged(true);
}


void FlyMode::setWallpaper(const KUrl& url) {
    
    setWallpaper(url.toLocalFile());
}


void FlyMode::setDirection(int index) {

    if( (_directionVertical && (index==0)) || (!_directionVertical && (index==1)) )
        return;
    
    _directionVertical = (index == 0);

    _pictureOffset = (_directionVertical) ? -_pixmap->height() : -_pixmap->width();   
    
    emit configNeedsSaving();
    emit settingsChanged(true);
}


void FlyMode::setSpeed(int speed) {

    if( speed < 1 )
        speed = 1;
    
    if( speed > SPEED_STEP_COUNT )
        speed = SPEED_STEP_COUNT;

    _pixelPerSecond = speed *SPEED_STEP;

    emit configNeedsSaving();
    emit settingsChanged(true);
}


void FlyMode::paint(QPainter *painter, const QRectF& exposedRect) {

    if( _stopPaint )
        return;
    
    // blit the background (saves all the per-pixel-products that blending does)
    painter->setCompositionMode(QPainter::CompositionMode_Source);
            
    // draw the background untransformed when possible;(saves lots of per-pixel-math)
    painter->resetTransform();

    painter->setClipRect(exposedRect);

    int width  = boundingRect().size().toSize().width();
    int height = boundingRect().size().toSize().height();

    // not using painter->drawTiledPixmap ( doesn't work well )
    if( _directionVertical ) { 

        int cw = (int)(width / (float)_pixmap->width() +1);
        int ch = (int)((height -_pictureOffset) / (float)_pixmap->height() +1);

        for( int h=0; h<ch; h++ ) {    
            for( int w=0; w<cw; w++ ) {
                painter->drawPixmap( QPointF(_pixmap->width() *w, _pixmap->height() *h +_pictureOffset), *_pixmap);
            }
        }
    }
    // direction horizontal
    else {
        int cw = (int)((width -_pictureOffset) / (float)_pixmap->width() +1);
        int ch = (int)(height / (float)_pixmap->height() +1);
            
        for( int h=0; h<ch; h++ ) {    
            for( int w=0; w<cw; w++ ) {
                painter->drawPixmap( QPointF(_pixmap->width() *w +_pictureOffset, _pixmap->height() *h), *_pixmap);
            }
        }
    }
}
 
 
#include "FlyMode.moc"